/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * Projektbeschreibung:
 * @title
 * @version 1.0
 * @copyright Copyright (c) 2004
 */

/**
 * History:
 *
 * $Log: AuswerteQuerschnitt.java,v $
 * Revision 1.1  2007/07/04 11:43:32  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.5  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.4  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/06/13 14:17:09  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/03/16 08:11:02  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:25:48  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:19  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp straße
 */
public class AuswerteQuerschnitt extends ConfigurationObjectWrapper {

	private static WeakHashMap<String, AuswerteQuerschnitt> __Cache = new WeakHashMap<>();

	private static ConfigurationObjectType __ConfigurationObjectType;
	private static AttributeGroup __AtgAuswerteQuerschnitt;

	private LangZeitZaehlStelle _LangZeitZaehlStelle;
	private MessQuerschnitt _MessQuerschnitt;
	private String _Bezeichnung;
	private String _Himmelsrichtung;

	public String getBezeichnung() {
		return _Bezeichnung;
	}

	public String getHimmelsrichtung() {
		return _Himmelsrichtung;
	}

	public MessQuerschnitt getMessQuerschnitt() {
		return _MessQuerschnitt;
	}

	private AuswerteQuerschnitt(final String pid)
			throws DataModellObjectNotFoundException, DataModellMismatchException {
		super(pid);
	}

	public AuswerteQuerschnitt(final ConfigurationObject confObj) throws DataModellMismatchException {
		super(confObj);
	}

	@Override
	protected void initData(final ConfigurationObject confObj) throws DataModellMismatchException {
		final Data oData = confObj.getConfigurationData(AuswerteQuerschnitt.__AtgAuswerteQuerschnitt);
		if (oData != null) {
			_MessQuerschnitt = new MessQuerschnitt(
					(ConfigurationObject) oData.getReferenceValue("Messquerschnitt").getSystemObject());
			_MessQuerschnitt.setAuswerteQuerschnitt(this);
			_Bezeichnung = oData.getTextValue("Bezeichnung").getText();
			if (oData.getUnscaledValue("Himmelsrichtung").isState()) {
				_Himmelsrichtung = oData.getUnscaledValue("Himmelsrichtung").getState().getName();
			}
		}
		AuswerteQuerschnitt.__Cache.put(this.getPID(), this);
	}

	/**
	 * gibt die Instance des AuswerteQuerschnittes mit der PID zurück. Es wird
	 * eine DataModellObjectNotFoundException geworfen, wenn die PID nicht
	 * vorhanden ist oder das zur PID gehörende Objekt nicht vom typ straße ist.
	 *
	 * @param pid
	 *            String
	 * @throws DataModellObjectNotFoundException
	 *             Instanz konnte nicht ermittelt werden
	 * @throws DataModellMismatchException
	 *             Instanz konnte nicht ermittelt werden
	 * @return Strasse
	 */
	public static AuswerteQuerschnitt getInstance(final String pid)
			throws DataModellMismatchException, DataModellObjectNotFoundException {

		AuswerteQuerschnitt retValue = AuswerteQuerschnitt.__Cache.get(pid);
		if (retValue == null) {
			retValue = new AuswerteQuerschnitt(pid);
		}
		return retValue;
	}

	/**
	 * gibt die DatenModellObjekte vom typ auswerteQuerschnitt zurück.
	 *
	 * @throws DataModellMismatchException
	 *             Instanz konnte nicht ermittelt werden
	 * @return Strasse[]
	 */
	public static AuswerteQuerschnitt[] getAll() throws DataModellMismatchException {

		final List<SystemObject> list = LogosClientDavConnection.getInstance().getDataModel().getType("typ.auswerteQuerschnitt")
				.getElements();
		final AuswerteQuerschnitt[] retValue = new AuswerteQuerschnitt[list.size()];
		for (int i = 0; i < list.size(); i++) {
			retValue[i] = new AuswerteQuerschnitt((ConfigurationObject) list.get(i));
		}
		return retValue;
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("AuswerteQuerschnitt -- PID: \"").append(getPID()).append("\"  ");
		sb.append("Name: \"").append(getName()).append("\"  ");
		sb.append("Bezeichnung: \"").append(_Bezeichnung).append("\"");
		return sb.toString();
	}

	public String getMultilineRepresentation() {
		return getMultilineRepresentation(0);
	}

	public String getMultilineRepresentation(final int indent) {
		final String lf = System.getProperty("line.separator");
		final StringBuffer sIndent = new StringBuffer();
		for (int i = 0; i < indent; i++) {
			sIndent.append(' ');
		}
		final StringBuffer sb = new StringBuffer();

		sb.append(sIndent).append("PID: \"").append(getPID()).append("\"").append(lf);
		sb.append(sIndent).append("Name: \"").append(getName()).append("\"").append(lf);
		sb.append(sIndent).append("Bezeichnung: \"").append(_Bezeichnung).append("\"").append(lf);
		sb.append(sIndent).append("MessQuerschnitt: ").append(lf);
		if (_MessQuerschnitt != null) {
			sb.append(_MessQuerschnitt.getMultilineRepresentation(indent + 2));
		} else {
			sb.append(sIndent).append("  kein Messquerschnitt vorhanden");
		}
		return sb.toString();
	}

	public void setLangZeitZaehlStelle(final LangZeitZaehlStelle lzz) {
		_LangZeitZaehlStelle = lzz;
	}

	/**
	 * gibt die Langzeizz&auml;hlstelle zur&uuml;ck, wenn dieses Objekt durch
	 * eine Langzeizz&auml;hlstelle erzeugt wurde.
	 *
	 * @return LangZeitZaehlStelle
	 */
	public LangZeitZaehlStelle getLangZeitZaehlStelle() {
		return _LangZeitZaehlStelle;
	}

	// public String getTreeString() {
	// StringBuffer sb = new StringBuffer();
	// if (_Bezeichnung.startsWith("GE")) {
	// sb.append("RFB
	// ").append(_LangZeitZaehlStelle.getInFernziel()).append('-').append(_LangZeitZaehlStelle.getGegenFernziel());
	// return sb.toString();
	// }
	// else {
	// if (_Bezeichnung.startsWith("IN")) {
	// sb.append("RFB
	// ").append(_LangZeitZaehlStelle.getGegenFernziel()).append('-').append(_LangZeitZaehlStelle.getInFernziel());
	// return sb.toString();
	// }
	// else {
	// return getName();
	// }
	// }
	// }

	@Override
	public String getTreeString() {
		final StringBuffer sb = new StringBuffer();
		if (_Bezeichnung.startsWith("GE")) {
			sb.append("RFB nach ").append(_LangZeitZaehlStelle.getGegenFernziel());
			return sb.toString();
		}
		if (_Bezeichnung.startsWith("IN")) {
			sb.append("RFB nach ").append(_LangZeitZaehlStelle.getInFernziel());
			return sb.toString();
		}
		return getName();
	}

	@Override
	ConfigurationObjectType getType() {
		if (AuswerteQuerschnitt.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			AuswerteQuerschnitt.__ConfigurationObjectType = (ConfigurationObjectType) model
					.getType("typ.auswerteQuerschnitt");
			AuswerteQuerschnitt.__AtgAuswerteQuerschnitt = model.getAttributeGroup("atg.auswerteQuerschnitt");
			final List<SystemObject> list = AuswerteQuerschnitt.__ConfigurationObjectType.getElements();
			model.getConfigurationData(list, AuswerteQuerschnitt.__AtgAuswerteQuerschnitt);
		}
		return AuswerteQuerschnitt.__ConfigurationObjectType;
	}

}
