/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

/**
 * History:
 *
 * $Log: BastExporter.java,v $
 * Revision 1.5  2007/08/22 08:49:07  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2007/07/26 08:01:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/04 11:48:42  Trier
 * Neuordnung
 *
 * Revision 1.2  2007/06/20 12:36:12  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2007/06/20 09:56:27  Trier
 * dvs alles
 *
 * Revision 1.8  2007/01/16 09:56:50  Trier
 * Erweiterung BastImport/Export
 * insbesondere die Unterstützung  unterschiedl. Aspekte
 *
 * Revision 1.7  2006/11/13 09:25:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.6  2005/09/22 06:56:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2005/09/13 09:14:57  Strey
 * Aktualisierung
 *
 * Revision 1.4  2005/02/28 14:40:47  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/02/18 10:09:51  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/31 15:16:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 14:40:01  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.ResultDataChannel;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.ResultDataProvider;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.AuswerteQuerschnitt;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.Fahrstreifen;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.MessQuerschnitt;

/**
 * erm&ouml;glicht die Ausgabe von Z&auml;hlstellendaten in das BASt-Format.
 *
 * @version 1.0
 */
public class BastExporter extends java.lang.Object implements ResultDataChannel {

	private static final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger LOGGER = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory
			.getLoggerFactory().getLogger();

	/**
	 * Konstante zum Erzeugen eines BAStExporters f&uuml;r die
	 * BAStBestandsbandformat der Version 2004
	 */
	public final static int BAST_FORMAT_V2004 = 0;

	/**
	 * Konstante zum Erzeugen eines BAStExporters f&uuml;r das in Sachsen-Anhalt
	 * verwendete BAStBestandsbandformat
	 */
	public final static int BAST_FORMAT_SA = 1;

	/**
	 * Konstante zum Erzeugen eines BAStExporters f&uuml;r das in
	 * Mecklenburg-Vorpommern verwendete BAStBestandsbandformat
	 */
	public final static int BAST_FORMAT_MV = 2;

	/**
	 * Konstante zum Erzeugen eines BAStExporters f&uuml;r die Ausgabe von
	 * AchslastDaten im BASt-Format Version 2004
	 */
	public final static int BAST_FORMAT_AXL = 3;

	/**
	 * Konstante zum Erzeugen eines BAStExporters f&uuml;r Testzwecke
	 */
	public final static int BAST_FORMAT_MV_TEST = 4;

	private static final int[] FIRST_MONTH_IN_QUARTAL = new int[] { Calendar.JANUARY, Calendar.APRIL, Calendar.JULY,
			Calendar.OCTOBER };

	private AbstractDataChannelBASt[] _Channels;
	private final ResultDataProvider _ResultDataProvider;

	private DataDescription _DataDescription;
	private final DataDescription _DataDescriptionAxl;

	private ChannelLogger _ChannelLogger;

	private Thread _Thread;

	private final int _Format;

	public BastExporter(final int format) throws Exception {
		final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
		_Format = format;
		switch (format) {
		case BAST_FORMAT_V2004: {
			_Channels = new AbstractDataChannelBASt[] { new DataChannelBAStV20() };
			break;
		}
		case BAST_FORMAT_SA: {
			_Channels = new AbstractDataChannelBASt[] { new DataChannelBASt_SA(DataChannelBAStV1_X.G_SATZ),
					new DataChannelBASt_SA(DataChannelBAStV1_X.K_SATZ),
					new DataChannelBASt_SA(DataChannelBAStV1_X.V_SATZ),
					new DataChannelBASt_SA(DataChannelBAStV1_X.M_SATZ) };
			break;
		}
		case BAST_FORMAT_MV: {
			_Channels = new AbstractDataChannelBASt[] { new DataChannelBASt_MV(DataChannelBAStV1_X.G_SATZ),
					new DataChannelBASt_MV(DataChannelBAStV1_X.K_SATZ),
					new DataChannelBASt_MV(DataChannelBAStV1_X.V_SATZ),
					new DataChannelBASt_MV(DataChannelBAStV1_X.M_SATZ) };
			break;
		}
		case BAST_FORMAT_MV_TEST: {
			_Channels = new AbstractDataChannelBASt[] { new DataChannelBASt_MV_Test(DataChannelBAStV1_X.G_SATZ),
					new DataChannelBASt_MV_Test(DataChannelBAStV1_X.K_SATZ),
					new DataChannelBASt_MV_Test(DataChannelBAStV1_X.V_SATZ),
					new DataChannelBASt_MV_Test(DataChannelBAStV1_X.M_SATZ) };
			break;
		}
		case BAST_FORMAT_AXL: {
			_Channels = new AbstractDataChannelBASt[] {
					new DataChannelAxlBAStV2004(DataChannelAxlBAStV2004.A_SATZ, DataChannelAxlBAStV2004.IN_RICHTUNG),
					new DataChannelAxlBAStV2004(DataChannelAxlBAStV2004.A_SATZ, DataChannelAxlBAStV2004.GEGEN_RICHTUNG),
					new DataChannelAxlBAStV2004(DataChannelAxlBAStV2004.G_SATZ, DataChannelAxlBAStV2004.IN_RICHTUNG),
					new DataChannelAxlBAStV2004(DataChannelAxlBAStV2004.G_SATZ,
							DataChannelAxlBAStV2004.GEGEN_RICHTUNG) };
			break;
		}
		}
		_ResultDataProvider = new ResultDataProvider();
		_ResultDataProvider.setIncrement(1, Calendar.HOUR);

		_ResultDataProvider.addChannel(this);

		/*
		 * for (int i=0; i<_Channels.length; i++) {
		 * _ResultDataProvider.addChannel(_Channels[i]); }
		 */

		AttributeGroup atg = model.getAttributeGroup("atg.verkehrsDatenLangZeitIntervall");
		Aspect aspect = model.getAspect("asp.messWertErsetzung");
		_DataDescription = new DataDescription(atg, aspect, (short) 0);

		atg = model.getAttributeGroup("atg.tlsAxlErgebnisMeldungAchsLastDatenVersion10");
		aspect = model.getAspect("asp.tlsAntwort");
		_DataDescriptionAxl = new DataDescription(atg, aspect, (short) 0);
	}

	public void setAspect(final Aspect asp) {
		final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
		final AttributeGroup atg = model.getAttributeGroup("atg.verkehrsDatenLangZeitIntervall");
		_DataDescription = new DataDescription(atg, asp);
	}

	/**
	 * setzt die Ziffer, mit der die Stuhndenz&auml;lung beginnt ( 0 oder 1; 1 ==
	 * Standard).
	 *
	 * @param firstHour
	 *            0 oder 1
	 */
	public void setFirstHourOfDay(final int firstHour) {
		if (_Format == BastExporter.BAST_FORMAT_MV_TEST) {
			for (final AbstractDataChannelBASt channel : _Channels) {
				((DataChannelBASt_MV_Test) channel).setFirstHourOfDay(firstHour);
			}
		}
	}

	/**
	 * bestimmt, ob am Ende einer Zeile ein Semikolon gesetzt werden soll (Standard
	 * == nein).
	 *
	 * @param endLineWithSemikolon
	 *            true, wenn am Zeilenende ein Semikolon gesetzt werden soll.
	 */
	public void setEndLineWithSemikolon(final boolean endLineWithSemikolon) {
		if (_Format == BastExporter.BAST_FORMAT_MV_TEST) {
			for (final AbstractDataChannelBASt channel : _Channels) {
				((DataChannelBASt_MV_Test) channel).setEndLineWithSemikolon(endLineWithSemikolon);
			}
		}
	}

	/**
	 * <p>
	 * exportiert die Daten der Z&auml;hlstelle.
	 * </p>
	 *
	 * <p>
	 * Ist das File ein vorhandenes Verzeichnis wird der Dateiname der Ausgabedatei
	 * durch den benutzten Channel bestimmt. Vorhandene Dateien werden
	 * &uuml;berschrieben.
	 * </p>
	 *
	 * <p>
	 * Ist das File kein vorhandenes Verzeichnis wird zur Augabe das File benutzt.
	 * Existstiert die Datei, wird diese &uuml;berschrieben.
	 *
	 * @param lzz
	 *            LangZeitZaehlStelle, dessen Daten ausgegeben werden sollen.
	 * @param fileOrDirectory
	 *            Verzeichnis in dem die Datei(en) geschrieben werden sollen, oder
	 *            Dateiname
	 * @param year
	 *            Jahr f&uuml;r das die Ausgabe erfolgen soll.
	 * @param month
	 *            Monat f&uuml;r den die Ausgabe erfolgen soll (Januar=0).
	 * @throws ChannelException
	 *             Fehler beim Zugriff auf den Channel
	 * @throws DataProviderException
	 *             Daten konnten nicht erfolgreich ausgegeben werden
	 */
	public void export(final LangZeitZaehlStelle lzz, final File fileOrDirectory, final int year, final int month)
			throws ChannelException, DataProviderException {
		export(lzz, fileOrDirectory, year, month, false);
	}

	/**
	 * <p>
	 * exportiert die Daten der Z&auml;hlstelle.
	 * </p>
	 *
	 * <p>
	 * Ist das File ein vorhandenes Verzeichnis wird der Dateiname der Ausgabedatei
	 * durch den benutzten Channel bestimmt. Vorhandene Dateien werden
	 * &uuml;berschrieben.
	 * </p>
	 *
	 * <p>
	 * Ist das File kein vorhandenes Verzeichnis wird zur Augabe das File benutzt.
	 * Existstiert die Datei, wird diese &uuml;berschrieben.
	 *
	 * @param lzz
	 *            LangZeitZaehlStelle, dessen Daten ausgegeben werden sollen.
	 * @param fileOrDirectory
	 *            Verzeichnis in dem die Datei(en) geschrieben werden sollen, oder
	 *            Dateiname
	 * @param year
	 *            Jahr f&uuml;r das die Ausgabe erfolgen soll.
	 * @param monthOrQuartal
	 *            Monat oder Quartal f&uuml;r den bzw. das die Ausgabe erfolgen
	 *            soll.
	 * @param isQuartal
	 *            true, wenn der Parameter monthOrYear als Quartal interpretiert
	 *            werden soll.
	 * @throws ChannelException
	 *             die Datenverarbeitung war nicht erfolgreich
	 * @throws DataProviderException
	 *             die Daten konnten nicht exportiert werden
	 */
	public void export(final LangZeitZaehlStelle lzz, final File fileOrDirectory, final int year,
			final int monthOrQuartal, final boolean isQuartal) throws ChannelException, DataProviderException {

		_Thread = Thread.currentThread();

		// Channels zurücksetzen
		if (fileOrDirectory.isDirectory()) {
			for (final AbstractDataChannelBASt channel : _Channels) {
				channel.reset(lzz, fileOrDirectory.getAbsolutePath(), year, monthOrQuartal, isQuartal);
			}
		} else {
			for (final AbstractDataChannelBASt channel : _Channels) {
				channel.reset(lzz, fileOrDirectory.getAbsolutePath());
			}
		}

		// ResultDataProvider ZeitBereich setzen
		final Calendar calender = Calendar.getInstance();
		calender.clear();
		calender.set(Calendar.YEAR, year);
		calender.set(Calendar.HOUR, 0);
		calender.set(Calendar.MINUTE, 0);
		calender.set(Calendar.MILLISECOND, 0);
		if (isQuartal) {
			calender.set(Calendar.MONTH, BastExporter.FIRST_MONTH_IN_QUARTAL[monthOrQuartal]);
			calender.set(Calendar.DAY_OF_MONTH, 1);
			_ResultDataProvider.setPeriod(calender, 3, Calendar.MONTH);
		} else {
			calender.set(Calendar.MONTH, monthOrQuartal);
			calender.set(Calendar.DAY_OF_MONTH, 1);
			_ResultDataProvider.setPeriod(calender, 1, Calendar.MONTH);
		}
		// System.err.println(" calender="+calender.getTime());

		if (_Format == BastExporter.BAST_FORMAT_AXL) {
			exportAxl(lzz);
		} else {
			export(lzz);
		}
	}

	/**
	 * <p>
	 * exportiert die Daten der Z&auml;hlstelle.
	 * </p>
	 *
	 * <p>
	 * Ist das File ein vorhandenes Verzeichnis wird der Dateiname der Ausgabedatei
	 * durch den benutzten Channel bestimmt. Vorhandene Dateien werden
	 * &uuml;berschrieben.
	 * </p>
	 *
	 * <p>
	 * Ist das File kein vorhandenes Verzeichnis wird zur Augabe das File benutzt.
	 * Existstiert die Datei, wird diese &uuml;berschrieben.
	 *
	 * @param lzz
	 *            LangZeitZaehlStelle, dessen Daten ausgegeben werden sollen.
	 * @param fileOrDirectory
	 *            Verzeichnis in dem die Datei(en) geschrieben werden sollen, oder
	 *            Dateiname.
	 * @param anfCalendar
	 *            Zeitpunkt, ab dem die Daten ausgegeben werden sollen.
	 * @param endCalendar
	 *            Zeitpunkt, bis zu dem die Daten ausgegeben werden sollen
	 *            (exclusiv).
	 * @throws ChannelException
	 *             die Daten wurden nicht erfolgreich verarbeitet
	 * @throws DataProviderException
	 *             export der Daten nicht erfolgreich
	 */
	public void export(final LangZeitZaehlStelle lzz, final File fileOrDirectory, final Calendar anfCalendar,
			final Calendar endCalendar) throws ChannelException, DataProviderException {

		export(lzz, fileOrDirectory, anfCalendar, endCalendar, null);
	}

	public void export(final LangZeitZaehlStelle lzz, final File fileOrDirectory, final Calendar anfCalendar,
			final Calendar endCalendar, final String klassifizierungsGruppe)
			throws ChannelException, DataProviderException {

		_Thread = Thread.currentThread();

		// Channels zurücksetzen
		if (fileOrDirectory.isDirectory()) {
			for (final AbstractDataChannelBASt channel : _Channels) {
				channel.reset(lzz, fileOrDirectory.getAbsolutePath(), anfCalendar.get(Calendar.YEAR),
						anfCalendar.get(Calendar.DAY_OF_YEAR), endCalendar.get(Calendar.DAY_OF_YEAR));
			}
		} else {
			for (final AbstractDataChannelBASt channel : _Channels) {
				channel.reset(lzz, fileOrDirectory.getAbsolutePath());
			}
		}

		// u.U. Klassifizierungsgruppe setzten
		if (klassifizierungsGruppe != null) {
			try {
				for (final AbstractDataChannelBASt channel : _Channels) {
					((DataChannelBAStV1_X) channel).setKlassifizierungsgruppeBAST(klassifizierungsGruppe);
				}
			} catch (final ClassCastException e) {
				throw new ChannelException("diese Methode ist nur für Channel vom Typ DataChannelBAStV1_X zulässig");
			}
		}

		// ResultDataProvider ZeitBereich setzen
		final long lDiffInMilllis = endCalendar.getTimeInMillis() - anfCalendar.getTimeInMillis();
		final int iDiffInMin = (int) (lDiffInMilllis / 1000 / 60);
		_ResultDataProvider.setPeriod(anfCalendar, iDiffInMin, Calendar.MINUTE);

		if (_Format == BastExporter.BAST_FORMAT_AXL) {
			exportAxl(lzz);
		} else {
			export(lzz);
		}
	}

	private void export(final LangZeitZaehlStelle lzz) throws DataProviderException, ChannelException {
		BastExporter.LOGGER.finer("export( Zählstelle\"%s\")", lzz);
		_ChannelLogger = new ChannelLogger(lzz.toString());

		for (final AbstractDataChannelBASt channel : _Channels) {
			channel.setChannelLogger(_ChannelLogger);
		}

		_ResultDataProvider.clearDataCombinations();
		final List<Fahrstreifen> list = getFS(lzz);
		if ((list != null) && (list.size() > 0)) {
			for (int i = 0; i < list.size(); i++) {
				_ResultDataProvider.addDataCombination(list.get(i).getConfigurationObject(), _DataDescription);
			}
			// _ResultDataProvider.checkSequence(System.out);
			_ResultDataProvider.processData();
		}
		for (final AbstractDataChannelBASt channel : _Channels) {
			channel.close();
		}
	}

	private void exportAxl(final LangZeitZaehlStelle lzz) throws DataProviderException, ChannelException {
		System.out.println("exportAxl( Zählstelle\"" + lzz + "\")");
		_ChannelLogger = new ChannelLogger(lzz.toString());

		for (final AbstractDataChannelBASt channel : _Channels) {
			channel.setChannelLogger(_ChannelLogger);
		}

		_ResultDataProvider.clearDataCombinations();

		final Fahrstreifen[] fsArray = lzz.getFahrstreifen();

		for (final Fahrstreifen element : fsArray) {
			try {
				// System.out.println("fsArray["+i+"]=" + fsArray[i].getPID() +
				// " " + fsArray[i].isAchslastErfassung());
				if (element.isAchslastErfassung()) {
					_ResultDataProvider.addDataCombination(element.getDEKanalAxl().getConfigurationObject(),
							_DataDescriptionAxl);
				}
			} catch (final Exception ex) {
				ex.printStackTrace();
				throw new ChannelException(ex);
			}
		}
		// _ResultDataProvider.checkSequence(System.out);
		_ResultDataProvider.processData();

		for (final AbstractDataChannelBASt channel : _Channels) {
			channel.close();
		}
	}

	/**
	 * bricht einen laufenden Export ab.
	 */
	public void interrupt() {
		if (_Thread != null) {
			System.err.println("interrupting: " + _Thread.getName());
			_ResultDataProvider.interrupt();
			if (_Thread != null) {
				_Thread.interrupt();
			}
			System.err.println("interrupted called: " + _Thread.getName());
		}
	}

	@Override
	public void close() {
		try {
			for (final AbstractDataChannelBASt channel : _Channels) {
				channel.close();
			}
		} catch (final ChannelException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * gibt die sortierten Fahrstreifen der Z&auml;hlstelle zur&uuml;ck.
	 *
	 * @param lzz
	 *            LangZeitZaehlStelle
	 * @return Liste mit den sortierten Fahrstreifen
	 */
	private List<Fahrstreifen> getFS(final LangZeitZaehlStelle lzz) {

		Fahrstreifen[] fsInArray = null;
		Fahrstreifen[] fsGegenArray = null;
		final AuswerteQuerschnitt inAuswertequerschnitt = lzz.getInAuswertequerschnitt();
		if (inAuswertequerschnitt != null) {
			final MessQuerschnitt mq = inAuswertequerschnitt.getMessQuerschnitt();
			if (mq != null) {
				fsInArray = mq.getFahrstreifen();
			}
		}
		final AuswerteQuerschnitt gegenAuswertequerschnitt = lzz.getGegenAuswertequerschnitt();
		if (gegenAuswertequerschnitt != null) {
			final MessQuerschnitt mq = gegenAuswertequerschnitt.getMessQuerschnitt();
			if (mq != null) {
				fsGegenArray = mq.getFahrstreifen();
			}
		}

		// die Fahrstreifen werden von rechts (HauptFS) nach links (ÜFS01,
		// ÜFS02..) zurückgegeben.
		// die Ausgabe in das BASt-Format erfolgt aber in der Reihenfolge
		// HFS 1.RFB, ÜFS01 1.RFB, ÜFS02 1.RFB, ... , ÜFS02 2.RFB, ÜFS01 2.RFB,
		// HFS 2.RFB
		final List<Fahrstreifen> fsList = new ArrayList<>();
		if (fsInArray != null) {
			for (final Fahrstreifen element : fsInArray) {
				fsList.add(element);
			}
		}
		if (fsGegenArray != null) {
			for (int i = (fsGegenArray.length - 1); i >= 0; i--) {
				fsList.add(fsGegenArray[i]);
			}
		}
		return fsList;
	}

	/**
	 * gibt die Gesamtl&auml;nge des Jobs zur&uuml;ck.
	 * 
	 * @param year
	 *            das Jahr
	 * @param monthOrQuartal
	 *            der Monat oder das Quartal
	 * @param isQuartal
	 *            wurde Monat oder Quartal angegeben
	 *
	 * @return int
	 */
	public static int calculateTaskLength(final int year, final int monthOrQuartal, final boolean isQuartal) {
		final Calendar calender = Calendar.getInstance();
		calender.clear();
		calender.set(Calendar.YEAR, year);
		calender.set(Calendar.HOUR, 0);
		calender.set(Calendar.MINUTE, 0);
		if (isQuartal) {
			calender.set(Calendar.MONTH, BastExporter.FIRST_MONTH_IN_QUARTAL[monthOrQuartal]);
			return ResultDataProvider.calculateTotalPeriodInMinutes(calender, Calendar.MONTH, 3);
		}
		calender.set(Calendar.DAY_OF_MONTH, 1);
		return ResultDataProvider.calculateTotalPeriodInMinutes(calender, Calendar.MONTH, 1);
	}

	/**
	 * gibt die Gesamtl&auml;nge des Jobs zur&uuml;ck.
	 * 
	 * @param calPeriodStart
	 *            der Anfangszeitpunkt der Periode
	 * @param calPeriodEnd
	 *            der Endzeitpunkt der Periode
	 *
	 * @return int
	 */
	public static int calculateTaskLength(final Calendar calPeriodStart, final Calendar calPeriodEnd) {
		return ResultDataProvider.calculateTotalPeriodInMinutes(calPeriodStart, calPeriodEnd);
	}

	/**
	 * gibt L&auml;nge des abgearbeiten Teils des Jobs zur&uuml;ck.
	 *
	 * @return int
	 */
	public int getTaskProgressValue() {
		return _ResultDataProvider.getProgressedPeriodInMinutes();
	}

	/**
	 * verarbeitet intern Datens&auml;tze; darf nicht au&szlig;erhalb der Klasse
	 * aufgerufen werden.
	 *
	 * @param rsArrayAtTime
	 *            ResultData[]
	 * @param calendar
	 *            Calendar
	 */
	@Override
	public void processRecord(final Dataset[] rsArrayAtTime, final Calendar calendar) throws ChannelException {

		try {
			// allg Check, ob Daten für alls FS vorhanden
			if (rsArrayAtTime == null) {
				_ChannelLogger.logWarning(calendar.getTime(), "keine Daten");
			} else {
				int iFSWithOutData = 0;
				for (final Dataset element : rsArrayAtTime) {
					if ((element == null) || (element.getData() == null)) {
						iFSWithOutData++;
					}
				}
				if (iFSWithOutData != 0) {
					if (iFSWithOutData == rsArrayAtTime.length) {
						_ChannelLogger.logWarning(calendar.getTime(), "alle Fahrstreifen ohne Daten");
					} else {
						_ChannelLogger.logWarning(calendar.getTime(),
								String.valueOf(iFSWithOutData) + " Fahrstreifen ohne Daten");
					}
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
		for (final AbstractDataChannelBASt channel : _Channels) {
			channel.processRecord(rsArrayAtTime, calendar);
		}
	}

	/**
	 * gibt den f&uuml;r die Protokollierung zust&auml;ndigen Logger zur&uuml;ck.
	 *
	 * @return ChannelLogger
	 */
	public ChannelLogger getLogger() {
		return _ChannelLogger;
	}

}
