/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

/**
 * History:
 *
 * $Log: DataChannelAxlBAStV2004.java,v $
 * Revision 1.6  2007/08/22 07:48:45  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2007/07/26 12:49:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2007/07/26 08:01:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/25 13:02:06  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/04 11:48:42  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:27  Trier
 * dvs alles
 *
 * Revision 1.2  2006/11/13 09:25:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/09/22 06:56:22  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 14:40:05  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.AuswerteQuerschnitt;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.Fahrstreifen;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;

/**
 * <p>
 * Channel zur Ausgabe der Verkehrsmengendaten in das BASt-Bestandsbandformat
 * Version 2004.
 * </p>
 *
 * <p>
 * Es wird die Attributegrupe atg.verkehrsDatenLangZeitIntervall mit dem Aspekt
 * asp.messWertErsetzung ausgegeben.
 * </p>
 * <p>
 * Die Unterstützung f&uuml;r die durch die BASt definierten Pr&uuml;fzeichen
 * ist der folgenen Tabelle zu entnehmen.
 * </p>
 * 
 * <table border=1 summary="Werte">
 * <tr>
 * <th align=left>Pr&uuml;fkenn- zeichen</th>
 * <th align=left>Beschreibung lt. BAST</th>
 * <th align=left>Bemerkung</th>
 * </tr>
 * <tr>
 * <td>-</td>
 * <td>korrekte, regelmäßige Werte</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>u</td>
 * <td>korrekte, aber unregelmäßige Werte (z.B. wg. Großverantstaltung)</td>
 * <td>wird nicht unterst&uuml;tzt</td>
 * </tr>
 * <tr>
 * <td>a</td>
 * <td>fehlende Werte (Ausfall)</td>
 * <td>wenn Status == "nicht ermittelbar"</td>
 * </tr>
 * <tr>
 * <td>d</td>
 * <td>fehlerhafte Werte</td>
 * <td>wenn G&uuml;te &lt;&gt; 1</td>
 * </tr>
 * <tr>
 * <td>s</td>
 * <td>Sch&auml;tzwert wegen fehlender Werte</td>
 * <td>wird nicht unterst&uuml;tzt</td>
 * </tr>
 * <tr>
 * <td>k</td>
 * <td>Sch&auml;tzwert wegen fehlerhafter Werte</td>
 * <td>wird nicht unterst&uuml;tzt</td>
 * </tr>
 * <tr>
 * <td>z</td>
 * <td>Zeitumstellung (auf MESZ bzw. auf MEZ)</td>
 * <td>&nbsp;</td>
 * </tr>
 * </table>
 *
 * <p>
 * Die fehlende Stunde bei der Umstellung von der MEZ zur MESZ wird mit 0-Werten
 * und Pr&uuml;kennzeichen z (Zeitumstellung) und der Statuskennung i
 * ausgegeben.
 *
 * Bei der Umstellung MESZ zur MEZ wird der 2. Datensatz für 02 Uhr ausgegeben.
 * </p>
 *
 * <p>
 * siehe auch:
 * {@link de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output.AbstractDataChannelBASt
 * AbstractDataChannelBASt}
 * </p>
 */
public class DataChannelAxlBAStV2004 extends AbstractDataChannelBASt {

	private static final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger LOGGER = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory
			.getLoggerFactory().getLogger();

	private static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyMMdd");
	private static DecimalFormat HOUR_FORMAT = new DecimalFormat("00");
	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r Achslasten
	 */
	public final static int A_SATZ = 10;
	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r Gesamtgewichte
	 */
	public final static int G_SATZ = 11;

	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r die IN-Richtung
	 */
	public final static int IN_RICHTUNG = 10;
	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r die GEGEN-Richtung
	 */
	public final static int GEGEN_RICHTUNG = 11;

	private int _Type;
	private int _Richtung;

	private static String HEADER_ROW03_ACHSLASTEN = "S 3 11 9  11 E D R E00.0_02.0 E02.0_04.0 E04.0_06.0 E06.0_08.0 E08.0_10.0 E10.0_11.5 E11.5_12.0 E12.0_13.0 E13.0_14.0 E14.0_15.0 E15.0undgr D00.0_04.0 D04.0_08.0 D08.0_12.0 D12.0_16.0 D16.0_18.0 D18.0_19.0 D19.0_20.0 D20.0_22.0 D22.0undgr R00.0_03.0 R03.0_06.0 R06.0_09.0 R09.0_12.0 R12.0_15.0 R15.0_18.0 R18.0_21.0 R21.0_24.0 R24.0_27.0 R27.0_30.0 R30.0undgr;";
	private static String HEADER_ROW03_GESAMTLASTEN = "S 3 8  1 10 1 6  1 LoA Lzg Bus LoA00.0_03.5 LoA03.5_07.5 LoA07.5_14.0 LoA14.0_20.0 LoA20.0_26.0 LoA26.0_32.0 LoA32.0_38.8 LoA38.0undgr LoA_ueberlad Lzg00.0_07.5 Lzg07.5_14.0 Lzg14.0_20.0 Lzg20.0_26.0 Lzg26.0_32.0 Lzg32.0_38.0 Lzg38.0_40.0 Lzg40.0_44.0 Lzg44.0_50.0 Lzg50.0undgr Lzg_ueberlad Bus00.0_07.5 Bus07.5_14.0 Bus14.0_20.0 Bus20.0_26.0 Bus26.0_32.0 Bus32.0undgr Bus_ueberlad;";

	private AuswerteQuerschnitt _AuswerteQuerschnitt;
	private String _Fernziel;
	private Fahrstreifen[] _FSArray;
	private Fahrstreifen[] _FSMitDatenerfassungArray;

	private String _ThirdLine;
	private ValueExtractor _ValueExtractor;

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link AbstractDataChannelBASt#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss.
	 * 
	 * @param type
	 *            Typ des Kanals
	 * @param richtung
	 *            verwendete Richtung
	 * @throws ChannelException
	 *             Kanal konnte nicht angelegt werden
	 */
	public DataChannelAxlBAStV2004(final int type, final int richtung) throws ChannelException {
		_Type = type;
		_Richtung = richtung;
		if (type == DataChannelAxlBAStV2004.A_SATZ) {
			_ValueExtractor = new ValueExtractorAchslasten();
			_ThirdLine = DataChannelAxlBAStV2004.HEADER_ROW03_ACHSLASTEN;
		} else {
			if (type == DataChannelAxlBAStV2004.G_SATZ) {
				_ValueExtractor = new ValueExtractorGesamtlasten();
				_ThirdLine = DataChannelAxlBAStV2004.HEADER_ROW03_GESAMTLASTEN;
			} else {
				throw new ChannelException(
						"falscher type bei der Initialisierung (DataChannelAxlBAStV2004.A_SATZ oder DataChannelAxlBAStV2004.G_SATZ)");
			}
		}
	}

	/**
	 * erzeugt die BASt-Format-Daten und spooled diese.
	 *
	 * <p>
	 * vor dem ersten Aufruf muss, der Channel vollst&auml; initialisiert sein.
	 * </p>
	 *
	 * @param rsArrayAtTime
	 *            ResultData[]
	 * @param calendar
	 *            Intervallbeginn der Datens&auml;tze
	 * @throws ChannelException
	 *             Daten konnten nicht erfolgreich verarbeitet werden
	 */
	@Override
	public void processRecord(final Dataset[] rsArrayAtTime, final Calendar calendar) throws ChannelException {

		final int actHour = calendar.get(Calendar.HOUR);
		// Prüfen, ob Umstellung MEZ auf MESZ (Sommerzeit), dann gibt es für den
		// letzten
		// Sonntag im März 2 Uhr keine Daten.
		// Eine Zeile muss eingefügt werden.
		if ((_LastHour == 1) && (actHour == 3)) {
			final StringBuffer sbLine = new StringBuffer(180);
			sbLine.append(DataChannelAxlBAStV2004.DATE_FORMAT.format(calendar.getTime())).append(' ');
			sbLine.append(DataChannelAxlBAStV2004.HOUR_FORMAT.format(calendar.get(Calendar.HOUR_OF_DAY) + 1))
					.append(":00");
			sbLine.setCharAt(6, 'i');
			// Uhrzeit auf 2 setzen
			sbLine.setCharAt(8, '3');
			sbLine.append(_ValueExtractor.extractData(null));
			this.spool(sbLine);
		}
		if ((rsArrayAtTime != null) && (rsArrayAtTime.length > 0)) {
			final StringBuffer sbLine = new StringBuffer(180);
			sbLine.append(DataChannelAxlBAStV2004.DATE_FORMAT.format(calendar.getTime())).append(' ');
			sbLine.append(DataChannelAxlBAStV2004.HOUR_FORMAT.format(calendar.get(Calendar.HOUR_OF_DAY) + 1))
					.append(":00");

			// Die 2. Stunde am letzten Sonntag im Oktober gibt es zweimal
			// die erste schmeissen wir weg.
			// Da nur in diesem Fall eine Stunde doppelt sein kann
			// prüfen wir nur _LastHour == actHour
			if (_LastHour == actHour) {
				this.deleteLastLine();
			}
			sbLine.append(_ValueExtractor.extractData(rsArrayAtTime));

			if (!AbstractDataChannelBASt.hasData(rsArrayAtTime)) {
				sbLine.setCharAt(6, 'i');
			}
			this.spool(sbLine);
		} else {
			if (rsArrayAtTime == null) {
				DataChannelAxlBAStV2004.LOGGER.finer("rsArrayAtTime is null");
			} else {
				DataChannelAxlBAStV2004.LOGGER.finer("rsArrayAtTime.size: %s", rsArrayAtTime.length);
			}
		}
		_LastHour = actHour;
	}

	@Override
	public void setZaehlstelle(final LangZeitZaehlStelle zaehlstelle) throws ChannelException {

		_ZaehlStelle = zaehlstelle;

		if (_Richtung == DataChannelAxlBAStV2004.IN_RICHTUNG) {
			_AuswerteQuerschnitt = _ZaehlStelle.getInAuswertequerschnitt();
			_Fernziel = _ZaehlStelle.getInFernziel();
		} else {
			_AuswerteQuerschnitt = _ZaehlStelle.getGegenAuswertequerschnitt();
			_Fernziel = _ZaehlStelle.getGegenFernziel();
		}
		_FSArray = _AuswerteQuerschnitt.getMessQuerschnitt().getFahrstreifen();
		final List<Fahrstreifen> list = new ArrayList<>();
		for (final Fahrstreifen element : _FSArray) {
			try {
				if (element.isAchslastErfassung()) {
					list.add(element);
				}
			} catch (final Exception e) {
				throw new ChannelException(e);
			}
		}
		_FSMitDatenerfassungArray = list.toArray(new Fahrstreifen[list.size()]);
	}

	private StringBuffer contructFirstPartOfFileName(final String path,
			final int year) {
		final StringBuffer sb = new StringBuffer(path.length() + 20);
		sb.append(path);
		sb.append(_ZaehlStelle.getBundesland().getAbbreviation());
		sb.append(_ZaehlStelle.getBastNummerAsString());
		if (_Type == DataChannelAxlBAStV2004.A_SATZ) {
			sb.append('a');
		} else {
			sb.append('g');
		}
		if (_Richtung == DataChannelAxlBAStV2004.IN_RICHTUNG) {
			sb.append('1');
		} else {
			sb.append('2');
		}

		String sYear = String.valueOf(year % 100);
		if (sYear.length() == 1) {
			sYear = '0' + sYear;
		}
		sb.append('.').append(AbstractDataChannelBASt.TWO_DIGIT_FORMAT.format(year % 100));
		return sb;
	}

	@Override
	protected String constructFilename(final LangZeitZaehlStelle zaehlStelle, final String path, final int year,
			final int startDayOfYear, final int endDayOfYear) {

		final StringBuffer sb = contructFirstPartOfFileName(path, year);
		sb.append('_').append(AbstractDataChannelBASt.THREE_DIGIT_FORMAT.format(startDayOfYear));
		sb.append('_').append(AbstractDataChannelBASt.THREE_DIGIT_FORMAT.format(endDayOfYear));
		sb.append(".txt");
		DataChannelAxlBAStV2004.LOGGER.finer("FileName: %s", sb);
		return sb.toString();
	}

	@Override
	protected String constructFilename(final LangZeitZaehlStelle zaehlStelle, final String path, final int year,
			final int monthOrQuartal, final boolean isQuartal) {

		final StringBuffer sb = contructFirstPartOfFileName(path, year);

		if (isQuartal) {
			sb.append(monthOrQuartal);
		} else {
			sb.append(AbstractDataChannelBASt.MONATE[monthOrQuartal]);
			DataChannelAxlBAStV2004.LOGGER.finer("FileName: %s", sb);
		}
		return sb.toString();
	}

	private String calculateFirstRow() {

		final StringBuffer sb = new StringBuffer(51);

		sb.append('H');
		sb.append(_ZaehlStelle.getLzzNummerAsString());
		sb.append(' ');
		sb.append(_ZaehlStelle.getBundesland().getKennungAsString());
		sb.append(' ');
		sb.append(_ZaehlStelle.getStrassenKlasse().getAbbreviation());
		sb.append(' ');
		try {
			sb.append(_ZaehlStelle.getStrasse().getNummer()).append(_ZaehlStelle.getStrasse().getZusatz());
		} catch (final Exception ex1) {
			ex1.printStackTrace();
		}
		while (sb.length() < 21) {
			sb.append(' ');
		}
		sb.append(_ZaehlStelle.getLzzName());
		sb.setLength(46);
		sb.append("V2.0;");
		return sb.toString();
	}

	private String calculateSecondRow() {

		final String sInHimmelsrichtung = _AuswerteQuerschnitt.getHimmelsrichtung();
		final int iFSCountGesamtRichtung = _FSArray.length;
		final int iFSCountMitDatenerfassung = _FSMitDatenerfassungArray.length;

		final StringBuffer sb = new StringBuffer("R");
		if (iFSCountGesamtRichtung < 10) {
			sb.append('0');
		}
		sb.append(String.valueOf(iFSCountGesamtRichtung));
		sb.append(' ');
		if (iFSCountMitDatenerfassung < 10) {
			sb.append('0');
		}
		sb.append(String.valueOf(iFSCountMitDatenerfassung));
		sb.append(' ');

		sb.append(_Fernziel);
		sb.setLength(27);
		sb.append(sInHimmelsrichtung);
		sb.append(';');
		return sb.toString();
	}

	@Override
	protected void writeHeader(final PrintWriter writer) {
		writer.println(calculateFirstRow());
		writer.println(calculateSecondRow());
		writer.println(_ThirdLine);
	}

	static final String format(final String s) {
		String retValue = s;
		switch (s.length()) {
		case 2:
			retValue = "    " + s;
			break;
		case 3:
			retValue = "   " + s;
			break;
		case 4:
			retValue = "  " + s;
			break;
		case 5:
			retValue = " " + s;
			break;
		default:
			break;
		}
		return retValue;
	}

	private interface ValueExtractor {
		String extractData(Dataset[] rsArray);
	}

	StringBuffer sumValues(final Data.Array dataArray, final int arrayLength) {
		final StringBuffer sb = new StringBuffer();
		int iSum = 0;
		for (int i = 0; (i < arrayLength); i++) {
			final Data.NumberValue nrValue = dataArray.getUnscaledValue(i);
			if (nrValue.isNumber()) {
				iSum = iSum + nrValue.intValue();
			} else {
				sb.append("    0d");
				return sb;
			}
		}
		sb.append(String.valueOf(iSum)).append('-');
		while (sb.length() < 6) {
			sb.insert(0, ' ');
		}
		return sb;
	}

	StringBuffer extractValues(final Data.Array dataArray, final int arrayLength) {
		final StringBuffer sb = new StringBuffer();
		for (int i = 0; i < arrayLength; i++) {
			final Data.NumberValue nrValue = dataArray.getUnscaledValue(i);
			if (nrValue.isNumber()) {
				final String sValue = nrValue.getValueText();
				sb.setLength(sb.length() + (5 - sValue.length()));
				sb.append(sValue).append('-');
			} else {
				sb.append("    0d");
			}
		}
		return sb;
	}

	StringBuffer extractValue(final Data.NumberValue nrValue) {
		final StringBuffer sb = new StringBuffer();
		if (nrValue.isNumber()) {
			final String sValue = nrValue.getValueText();
			sb.setLength(sb.length() + (5 - sValue.length()));
			sb.append(sValue).append('-');
		} else {
			sb.append("    0d");
		}
		return sb;
	}

	/**
	 * extrahiert die Werte aus den ResultData und baut einen String mit den Werten
	 * f&uuml;r die Achslasten auf.
	 */
	private class ValueExtractorAchslasten implements ValueExtractor {

		@Override
		public String extractData(final Dataset[] rsArray) {
			// Einzelachsen E => 11 KLassen
			// Doppelachsen D => 9 KLassen
			// Dreifachachsen R => 11 KLassen
			final StringBuffer sb = new StringBuffer();
			if ((rsArray != null) && (rsArray.length > 0)) {
				for (int i = 0; i < _FSMitDatenerfassungArray.length; i++) {
					int rsNr = i;
					if (_Richtung == DataChannelAxlBAStV2004.GEGEN_RICHTUNG) {
						rsNr = rsArray.length - i - 1;
					}
					final Dataset rs = rsArray[rsNr];
					if ((rs != null) && (rs.getData() != null)) {
						final Data data = rs.getData();
						sb.append(sumValues(data.getArray("AnzEinzelachslastKlasse"), 11));
						sb.append(sumValues(data.getArray("AnzDoppelachslastKlasse"), 9));
						sb.append(sumValues(data.getArray("AnzDreifachachslastKlasse"), 11));
					} else {
						for (int nr = 0; (nr < (3)); nr++) {
							sb.append("    0a");
						}
					}
				}
				for (int i = 0; i < _FSMitDatenerfassungArray.length; i++) {
					int rsNr = i;
					if (_Richtung == DataChannelAxlBAStV2004.GEGEN_RICHTUNG) {
						rsNr = rsArray.length - i - 1;
					}
					final Dataset rs = rsArray[rsNr];
					if ((rs != null) && (rs.getData() != null)) {
						final Data data = rs.getData();
						sb.append(extractValues(data.getArray("AnzEinzelachslastKlasse"), 11));
						sb.append(extractValues(data.getArray("AnzDoppelachslastKlasse"), 9));
						sb.append(extractValues(data.getArray("AnzDreifachachslastKlasse"), 11));
					} else {
						for (int nr = 0; (nr < (11 + 9 + 11)); nr++) {
							sb.append("    0a");
						}
					}
				}
			} else {
				for (final Fahrstreifen element : _FSMitDatenerfassungArray) {
					for (int nr = 0; (nr < (3)); nr++) {
						sb.append("    0a");
					}
					for (int nr = 0; (nr < (11 + 9 + 11)); nr++) {
						sb.append("    0a");
					}
				}
			}
			return sb.toString();
		}
	}

	/**
	 * <p>
	 * extrahiert die Werte aus den ResultData und baut einen String mit den Werten
	 * f&uuml;r die Gesamtlasten auf.
	 * </p>
	 *
	 * <pre>
	 *   1..n Fahrstreifen:
	 *              Anzahl LoA
	 *              Anzahl Lzg
	 *              Anzahl Bus
	 *   1..n Fahrstreifen:
	 *              LoA =&gt;  8 KLassen
	 *              &Uuml;berladung LoA
	 *              Lzg =&gt; 10 KLassen
	 *              &Uuml;berladung Lzg
	 *              Bus =&gt;  6 KLassen
	 *              &Uuml;berladung Bus
	 * </pre>
	 */
	private class ValueExtractorGesamtlasten implements ValueExtractor {

		@Override
		public String extractData(final Dataset[] rsArray) {
			//
			final StringBuffer sb = new StringBuffer();
			if ((rsArray != null) && (rsArray.length > 0)) {
				for (int i = 0; i < _FSMitDatenerfassungArray.length; i++) {
					int rsNr = i;
					if (_Richtung == DataChannelAxlBAStV2004.GEGEN_RICHTUNG) {
						rsNr = rsArray.length - i - 1;
					}
					final Dataset rs = rsArray[rsNr];
					if ((rs != null) && (rs.getData() != null)) {
						final Data data = rs.getData();
						sb.append(sumValues(data.getArray("AnzGewichtsKlasseLkw"), 8));
						sb.append(sumValues(data.getArray("AnzGewichtsKlasseLkwA"), 10));
						sb.append(sumValues(data.getArray("AnzGewichtsKlasseBusse"), 6));
					} else {
						for (int nr = 0; (nr < 3); nr++) {
							sb.append("    0a");
						}
					}
				}
				for (int i = 0; i < _FSMitDatenerfassungArray.length; i++) {
					int rsNr = i;
					if (_Richtung == DataChannelAxlBAStV2004.GEGEN_RICHTUNG) {
						rsNr = rsArray.length - i - 1;
					}
					final Dataset rs = rsArray[rsNr];
					if ((rs != null) && (rs.getData() != null)) {
						final Data data = rs.getData();
						sb.append(extractValues(data.getArray("AnzGewichtsKlasseLkw"), 8));
						sb.append(extractValue(data.getUnscaledValue("AnzÜberladungLkw")));
						sb.append(extractValues(data.getArray("AnzGewichtsKlasseLkwA"), 10));
						sb.append(extractValue(data.getUnscaledValue("AnzÜberladungLkwA")));
						sb.append(extractValues(data.getArray("AnzGewichtsKlasseBusse"), 6));
						sb.append(extractValue(data.getUnscaledValue("AnzÜberladungBusse")));
					} else {
						// 8Lkw,1ÜlbadgLkw,10LkwA,1ÜbladgLkwA,6Bus,1ÜbladgBus
						for (int nr = 0; (nr < (8 + 10 + 6 + 3)); nr++) {
							sb.append("    0a");
						}
					}
				}
			} else {
				for (final Fahrstreifen element : _FSMitDatenerfassungArray) {
					for (int nr = 0; (nr < 3); nr++) {
						sb.append("    0a");
					}
					for (int nr = 0; (nr < (8 + 10 + 6 + 3)); nr++) {
						sb.append("    0a");
					}
				}
			}
			return sb.toString();
		}
	}

}
